@@ -1,7 +1,3 @@
-# Import the common functions.
-source('R/common.R')
-source('R/supervised-model-deployment.R')
-
 #' Deploy a production-ready predictive Random Forest model
 #'
 #' @description This step allows one to
@@ -22,20 +18,67 @@ source('R/supervised-model-deployment.R')
 #' @import RODBC
 #' @param type The type of model (either 'regression' or 'classification')
 #' @param df Dataframe whose columns are used for calc.
-#' @param grainCol The data frame's column that has IDs pertaining to the grain
+#' @param grainCol Optional. The dataframe's column that has IDs pertaining to 
+#' the grain. No ID columns are truly needed for this step.
 #' @param personCol The data frame's columns that represents the patient/person
-#' @param testWindowCol This column dictates the split between model training and
-#' test sets. Those rows with zeros in this column indicate the training set
-#' while those that have ones indicate the test set
-#' @param predictedCol Column that you want to predict.
+#' @param testWindowCol Y or N. This column dictates the split between model 
+#' training and test sets. Those rows with N in this column indicate the 
+#' training set while those that have Y indicate the test set
+#' @param predictedCol Column that you want to predict. If you're doing
+#' classification then this should be Y/N.
 #' @param impute For training df, set all-column imputation to F or T.
 #' This uses mean replacement for numeric columns
 #' and most frequent for factorized columns.
 #' F leads to removal of rows containing NULLs.
 #' @param debug Provides the user extended output to the console, in order
 #' to monitor the calculations throughout. Use T or F.
+#' @export
 #' @seealso \code{\link{healthcareai}}
 #' @examples
+#' 
+# #### Example using csv data ####
+#' ptm <- proc.time()
+#' library(healthcareai)
+#'
+#' # setwd('C:/Yourscriptlocation/Useforwardslashes') # Uncomment if using csv
+#' 
+#' # Can delete this line in your work
+#' csvfile <- system.file("extdata", 
+#'                        "HCRDiabetesClinical.csv", 
+#'                        package = "healthcareai")
+#'
+#' # Replace csvfile with 'path/file'
+#' df <- read.csv(file = csvfile, 
+#'                header = TRUE, 
+#'                na.strings = c("NULL", "NA", ""))
+#'
+#' head(df)
+#' str(df)
+#'
+#' p <- SupervisedModelDeploymentParams$new()
+#' p$type <- "classification"
+#' p$df <- df
+#' p$grainCol <- "PatientEncounterID"
+#' p$personCol <- "PatientID"
+#' p$testWindowCol <- "InTestWindowFLG"
+#' p$predictedCol <- "ThirtyDayReadmitFLG"
+#' p$impute <- TRUE
+#' p$debug <- FALSE
+#' p$useSavedModel <- FALSE
+#' p$cores <- 1
+#' p$writeToDB <- FALSE
+#'
+#' dLMM <- LinearMixedModelDeployment$new(p)
+#' dLMM$deploy()
+#' 
+#' df <- dLMM$getOutDf()
+#' # Write to CSV (or JSON, MySQL, etc) using R syntax
+#' # write.csv(df,'path/predictionsfile.csv')
+#' 
+#' print(proc.time() - ptm)
+#'
+#' \donttest{
+#'   
 #' #### Classification example using diabetes data ####
 #' # This example requires you to first create a table in SQL Server
 #' # If you prefer to not use SAMD, execute this in SSMS to create output table:
@@ -46,8 +89,6 @@ source('R/supervised-model-deployment.R')
 #' # Factor1TXT varchar(255), Factor2TXT varchar(255), Factor3TXT varchar(255)
 #' # )
 #'
-#' \donttest{
-#' #### This example is specific to Windows and is not tested. 
 #' # setwd('C:/Yourscriptlocation/Useforwardslashes') # Uncomment if using csv
 #' ptm <- proc.time()
 #' library(healthcareai)
@@ -76,14 +117,15 @@ source('R/supervised-model-deployment.R')
 #' df <- selectData(connection.string, query)
 #'
 #' head(df)
+#' str(df)
 #'
 #' p <- SupervisedModelDeploymentParams$new()
 #' p$type <- "classification"
 #' p$df <- df
 #' p$grainCol <- "PatientEncounterID"
+#' p$personCol <- "PatientID"
 #' p$testWindowCol <- "InTestWindowFLG"
 #' p$predictedCol <- "ThirtyDayReadmitFLG"
-#' p$personCol <- "PatientID"
 #' p$impute <- FALSE
 #' p$debug <- FALSE
 #' p$useSavedModel <- FALSE
@@ -95,6 +137,9 @@ source('R/supervised-model-deployment.R')
 #' lMM$deploy()
 #'
 #' print(proc.time() - ptm)
+#' }
+#' 
+#' \donttest{
 #'
 #' #### Regression example using diabetes data ####
 #' # This example requires you to first create a table in SQL Server
@@ -134,14 +179,15 @@ source('R/supervised-model-deployment.R')
 #' df <- selectData(connection.string, query)
 #'
 #' head(df)
+#' str(df)
 #'
 #' p <- SupervisedModelDeploymentParams$new()
 #' p$type <- "regression"
 #' p$df <- df
 #' p$grainCol <- "PatientEncounterID"
+#' p$personCol <- "PatientID"
 #' p$testWindowCol <- "InTestWindowFLG"
 #' p$predictedCol <- "A1CNBR"
-#' p$personCol <- "PatientID"
 #' p$impute <- TRUE
 #' p$debug <- FALSE
 #' p$useSavedModel <- FALSE
@@ -154,8 +200,6 @@ source('R/supervised-model-deployment.R')
 #'
 #' print(proc.time() - ptm)
 #' }
-#'
-#' @export
 
 LinearMixedModelDeployment <- R6Class("LinearMixedModelDeployment",
 
@@ -166,16 +210,20 @@ LinearMixedModelDeployment <- R6Class("LinearMixedModelDeployment",
   private = list(
 
     # variables
-    coefficients = NULL,
-    multiplyRes = NULL,
-    orderedFactors = NULL,
-    predictedValsForUnitTest = NULL,
+    coefficients = NA,
+    multiplyRes = NA,
+    orderedFactors = NA,
+    predictedValsForUnitTest = NA,
+    outDf = NA,
 
     # functions
     connectDataSource = function() {
       odbcCloseAll()
-      # Convert the connection string into a real connection object.
-      self$params$sqlConn <- odbcDriverConnect(self$params$sqlConn)
+      
+      if (isTRUE(self$params$writeToDB)) {
+        # Convert the connection string into a real connection object.
+        self$params$sqlConn <- odbcDriverConnect(self$params$sqlConn)
+      }
     },
 
     closeDataSource = function() {
@@ -324,10 +372,10 @@ LinearMixedModelDeployment <- R6Class("LinearMixedModelDeployment",
     },
 
     saveDataIntoDb = function() {
-      dtStamp <- as.POSIXlt(Sys.time(), "GMT")
+      dtStamp <- as.POSIXlt(Sys.time())
 
       # Combine grain.col, prediction, and time to be put back into SAM table
-      outdf <- data.frame(
+      private$outDf <- data.frame(
         0,                                 # BindingID
         'R',                               # BindingNM
         dtStamp,                           # LastLoadDTS
@@ -341,7 +389,7 @@ LinearMixedModelDeployment <- R6Class("LinearMixedModelDeployment",
       } else if (self$params$type == 'regression') {
         predictedResultsName <- "PredictedValueNBR"
       }
-      colnames(outdf) <- c(
+      colnames(private$outDf) <- c(
         "BindingID",
         "BindingNM",
         "LastLoadDTS",
@@ -353,26 +401,28 @@ LinearMixedModelDeployment <- R6Class("LinearMixedModelDeployment",
       )
 
       if (isTRUE(self$params$debug)) {
-        print('Dataframe going to SQL Server:')
-        print(str(outdf))
+        print('Dataframe with predictions:')
+        print(str(private$outDf))
       }
-
-      # Save df to table in SAM database
-      out <- sqlSave(
-        channel = self$params$sqlConn,
-        dat = outdf,
-        tablename = self$params$destSchemaTable,
-        append = T,
-        rownames = F,
-        colnames = F,
-        safer = T,
-        nastring = NULL,
-        verbose = self$params$debug
-      )
-
-      # Print success if insert was successful
-      if (out == 1) {
-        print('SQL Server insert was successful')
+      
+      if (isTRUE(self$params$writeToDB)) {
+        # Save df to table in SAM database
+        out <- sqlSave(
+          channel = self$params$sqlConn,
+          dat = private$outDf,
+          tablename = self$params$destSchemaTable,
+          append = T,
+          rownames = F,
+          colnames = F,
+          safer = T,
+          nastring = NULL,
+          verbose = self$params$debug
+        )
+  
+        # Print success if insert was successful
+        if (out == 1) {
+          print('SQL Server insert was successful')
+        }
       }
     }
   ),
@@ -504,6 +554,11 @@ LinearMixedModelDeployment <- R6Class("LinearMixedModelDeployment",
     #Get predicted values
     getPredictedValsForUnitTest = function() {
       return(private$predictedValsForUnitTest)
+    },
+    
+    # Surface outDf as attribute for export to Oracle, MySQL, etc
+    getOutDf = function() {
+      return(private$outDf)
     }
   )
 )
